/*=========================================================================

  Program:   Insight Segmentation & Registration Toolkit
  Module:    $RCSfile: itkMaximumDecisionRule.h,v $
  Language:  C++
  Date:      $Date: 2003/09/10 14:29:15 $
  Version:   $Revision: 1.6 $

  Copyright (c) Insight Software Consortium. All rights reserved.
  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#ifndef __itkMaximumDecisionRule_h
#define __itkMaximumDecisionRule_h

#include "itkWin32Header.h"

#include <vector>
#include "itkDecisionRuleBase.h"

namespace itk{ 

/** \class MaximumDecisionRule
 *  \brief A Decision rule that choose the class of which discriminant
 *  score is the largest.
 */
 
class ITKCommon_EXPORT MaximumDecisionRule : 
    public DecisionRuleBase
{
public:
  /** Standard class typedefs */ 
  typedef MaximumDecisionRule Self ;
  typedef DecisionRuleBase Superclass;
  typedef SmartPointer<Self> Pointer;
  typedef SmartPointer<const Self> ConstPointer;

  /** Run-time type information (and related methods) */
  itkTypeMacro(MaximumDecisionRule, DecisionRuleBase);

  /** Standard New() method support */
  itkNewMacro(Self) ;

  unsigned int Evaluate(std::vector< double > &discriminantScores) ;

protected:
  MaximumDecisionRule() {}
  virtual ~MaximumDecisionRule() {}
  void PrintSelf(std::ostream& os, Indent indent) const ;
} ; // end of class

inline unsigned int MaximumDecisionRule::Evaluate(std::vector< double > 
                                                  &discriminantScores)
{
  double max = discriminantScores[0] ;
  unsigned int maxIndex = 0 ;
  unsigned int i ;
  for (i = 1 ; i < discriminantScores.size() ; i++)
    {
    if (discriminantScores[i] > max) 
      {
      max = discriminantScores[i] ;
      maxIndex = i ;
      }
    }
  return maxIndex ;
}

} // end of namespace itk

#endif







